/*
 * All files in the distribution of JTB, The Java Tree Builder are
 * Copyright 1997, 1998 by the Purdue Research Foundation of Purdue
 * University.  All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted
 * provided that this entire copyright notice is duplicated in all
 * such copies, and that any documentation, announcements, and
 * other materials related to such distribution and use acknowledge
 * that the software was developed at Purdue University, West Lafayette,
 * Indiana by Kevin Tao and Jens Palsberg.  No charge may be made
 * for copies, derivations, or distributions of this material
 * without the express written consent of the copyright holder.
 * Neither the name of the University nor the name of the author
 * may be used to endorse or promote products derived from this
 * material without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR ANY PARTICULAR PURPOSE.
 */

package EDU.purdue.jtb.misc;

import java.util.*;

/**
 * Class AutoClasses simply contains string representations of the automatic
 * classes and their visitors.
 */
class AutoClasses {
   static final String packageName = Globals.nodePackage;

   static String getNodeClassStr() {
      return
         "package " + packageName + ";\n\n" +
         "import java.util.*;\n\n" +
         "/**\n" +
         " * The interface which all syntax tree classes must implement.\n" +
         " */\n" +
         "public interface Node extends java.io.Serializable {\n" +
         "   public void accept(" + Globals.visitorPackage + ".Visitor v);\n\n"+
         "   // It is the responsibility of each implementing class to call\n" +
         "   // setParent() on each of its child Nodes.\n" +
         "   public void setParent(Node n);\n" +
         "   public Node getParent();\n" +
         "   public void setChildAt(Node n, int i);\n"+
         "   public Vector getChildren();\n" +
         "}\n";
   }

   static String getNodeListInterfaceClassStr() {
      return
         "package " + packageName + ";\n\n" +
         "/**\n" +
         " * The interface which NodeList, NodeListOptional, and NodeSequence\n" +
         " * implement.\n" +
         " */\n" +
         "public interface NodeListInterface extends Node {\n" +
         "   public void addNode(Node n);\n" +
         "   public Node elementAt(int i);\n" +
         "   public java.util.Enumeration elements();\n" +
         "   public int size();\n" +
         "}\n";
   }

   static String getNodeChoiceClassStr() {
      StringBuffer buf = new StringBuffer(
         "package " + packageName + ";\n\n" +
         "import java.util.*;\n\n" +
         "/**\n" +
         " * Represents a grammar choice, e.g. ( A | B )\n" +
         " */\n");

      if ( Globals.superClassname == null )
         buf.append("public class NodeChoice implements Node {\n");
      else
         buf.append("public class NodeChoice extends "+Globals.superClassname+" implements Node {\n");

      buf.append(
         "   public NodeChoice(Node node) {\n" +
         "      this(node, -1);\n" +
         "   }\n\n" +
         "   public NodeChoice(Node node, int whichChoice) {\n" +
         "      choice = node;\n" +
         "      if (choice != null) choice.setParent(this);\n" +
         "      which = whichChoice;\n" +
         "   }\n\n" +
         "   public void accept(" + Globals.visitorPackage + ".Visitor v) {\n" +
         "      choice.accept(v);\n" +
         "   }\n\n" +
         "   public void setParent(Node n) { parent = n; }\n" +
         "   public Node getParent() { \n" +
         "      if (parent == null) throw new Error(\"Parent not set.\");\n"+
         "      return parent; \n"+
         "   }\n\n" +
         "   public void setChildAt(Node n, int i) { \n" +
         "      if (i != 0) throw new Error(\"Invalid Index.\"); \n" +
         "      choice = n ; \n" +
         "      if (n != null) n.setParent(this); \n" +
         "   }\n" +
         "   public Vector getChildren() { \n" +
         "      Vector vector = new Vector(1); \n" +
         "      vector.addElement(choice); \n" +
         "      return vector; \n" +
         "   }\n\n" +
         "   private Node parent;\n" +
         "   public Node choice;\n" +
         "   public int which;\n" +
         "}\n");

       return buf.toString();
   }

   static String getNodeListClassStr() {
      StringBuffer buf = new StringBuffer(
         "package " + packageName + ";\n\n" +
         "import java.util.*;\n\n" +
         "/**\n" +
         " * Represents a grammar list, e.g. ( A )+\n" +
         " */\n");

      if ( Globals.superClassname == null )
         buf.append("public class NodeList implements NodeListInterface {\n");
      else
         buf.append("public class NodeList extends "+Globals.superClassname+" implements NodeListInterface {\n");

      buf.append(
         "   public NodeList() {\n" +
         "      nodes = new Vector();\n" +
         "   }\n\n" +
         "   public NodeList(Node firstNode) {\n" +
         "      nodes = new Vector();\n" +
         "      addNode(firstNode);\n" +
         "   }\n\n" +
         "   public void addNode(Node n) {\n" +
         "      nodes.addElement(n);\n" +
         "      if (n != null) \n" +
         "         n.setParent(this);\n" +
         "      else \n"+
         "         throw new Error(\"Node is null.\"); \n" +
         "   }\n\n" +
         "   public Enumeration elements() { return nodes.elements(); }\n" +
         "   public Node elementAt(int i)  { return (Node)nodes.elementAt(i); }\n"+
         "   public int size()             { return nodes.size(); }\n\n" +
         "   public void accept(" + Globals.visitorPackage + ".Visitor v) {\n");

      if ( Globals.oldVisitNames )
         buf.append("      v.visitNodeList(this);\n");
      else
         buf.append("      v.visit(this);\n");

      buf.append(
         "   }\n\n" +
         "   public void setParent(Node n) { parent = n; }\n" +
         "   public Node getParent() { \n" +
         "      if (parent == null) throw new Error(\"Parent not set.\");\n"+
         "      return parent; \n"+
         "   }\n\n" +
         "   public void setChildAt(Node n, int i) { \n" +
         "      nodes.setElementAt(n,i); \n" +
         "      if (n != null) n.setParent(this); \n" +
         "   }\n" +
         "   public Vector getChildren() { \n" +
         "      return nodes; \n" +
         "   }\n\n" +
         "   private Node parent;\n" +
         "   public Vector nodes;\n" +
         "}\n");

      return buf.toString();
   }

   static String getNodeListOptionalClassStr() {
      StringBuffer buf = new StringBuffer(
         "package " + packageName + ";\n\n" +
         "import java.util.*;\n\n" +
         "/**\n" +
         " * Represents an optional grammar list, e.g. ( A )*\n" +
         " */\n");

      if ( Globals.superClassname == null )
         buf.append("public class NodeListOptional implements NodeListInterface {\n");
      else
         buf.append("public class NodeListOptional extends "+Globals.superClassname+" implements NodeListInterface {\n");

      buf.append(
         "   public NodeListOptional() {\n" +
         "      nodes = new Vector();\n" +
         "   }\n\n" +
         "   public NodeListOptional(Node firstNode) {\n" +
         "      nodes = new Vector();\n" +
         "      addNode(firstNode);\n" +
         "   }\n\n" +
         "   public void addNode(Node n) {\n" +
         "      nodes.addElement(n);\n" +
         "      if (n != null) \n" +
         "         n.setParent(this);\n" +
         "      else \n"+
         "         throw new Error(\"Node is null.\"); \n" +
         "   }\n\n" +
         "   public Enumeration elements() { return nodes.elements(); }\n" +
         "   public Node elementAt(int i)  { return (Node)nodes.elementAt(i); }\n"+
         "   public int size()             { return nodes.size(); }\n" +
         "   public boolean present()      { return nodes.size() != 0; }\n\n" +
         "   public void accept(" + Globals.visitorPackage + ".Visitor v) {\n");

      if ( Globals.oldVisitNames )
         buf.append("      v.visitNodeListOptional(this);\n");
      else
         buf.append("      v.visit(this);\n");

      buf.append(
         "   }\n\n" +
         "   public void setParent(Node n) { parent = n; }\n" +
         "   public Node getParent() { \n" +
         "      if (parent == null) throw new Error(\"Parent not set.\");\n"+
         "      return parent; \n"+
         "   }\n\n" +
         "   public void setChildAt(Node n, int i) { \n" +
         "      nodes.setElementAt(n, i); \n" +
         "      if (n != null) n.setParent(this); \n" +
         "   }\n" +
         "   public Vector getChildren() { \n" +
         "      return nodes; \n" +
         "   }\n\n" +
         "   private Node parent;\n" +
         "   public Vector nodes;\n" +
         "}\n");

      return buf.toString();
   }

   static String getNodeOptionalClassStr() {
      StringBuffer buf = new StringBuffer(
         "package " + packageName + ";\n\n" +
         "import java.util.*;\n\n" +
         "/**\n" +
         " * Represents an grammar optional node, e.g. ( A )? or [ A ]\n" +
         " */\n");

      if ( Globals.superClassname == null )
         buf.append("public class NodeOptional implements Node {\n");
      else
         buf.append("public class NodeOptional extends "+Globals.superClassname+" implements Node {\n");

      buf.append(
         "   public NodeOptional() {\n" +
         "      node = null;\n" +
         "   }\n\n" +
         "   public NodeOptional(Node n) {\n" +
         "      addNode(n);\n" +
         "   }\n\n" +
         "   public void addNode(Node n)  {\n" +
         "      if ( node != null)                // Oh oh!\n" +
         "         throw new Error(\"Attempt to set optional node twice\");\n\n" +
         "      node = n;\n" +
         "      if (n != null) \n" +
         "         n.setParent(this);\n" +
         "      else \n"+
         "         throw new Error(\"Node is null.\"); \n" +
         "   }\n\n" +
         "   public void accept(" + Globals.visitorPackage + ".Visitor v) {\n");

      if ( Globals.oldVisitNames )
         buf.append("      v.visitNodeOptional(this);\n");
      else
         buf.append("      v.visit(this);\n");

      buf.append(
         "   }\n" +
         "   public boolean present()   { return node != null; }\n\n" +
         "   public void setParent(Node n) { parent = n; }\n" +
         "   public Node getParent() { \n" +
         "      if (parent == null) throw new Error(\"Parent not set.\");\n"+
         "      return parent; \n"+
         "   }\n\n" +
         "   public void setChildAt(Node n, int i) { \n" +
         "      if (i != 0) throw new Error(\"Invalid Index.\"); \n" +
         "      node = n ; \n" +
         "      if (n != null) n.setParent(this); \n" +
         "   }\n" +
         "   public Vector getChildren() { \n" +
         "      Vector vector = new Vector(1); \n" +
         "      if (node != null) { \n"+
	 "         vector.addElement(node); \n" +
         "         return vector; \n" +
	 "      } \n" +
	 "      else \n"+
	 "         throw new Error(\"Empty NodeOptional.\"); \n" +
         "   }\n\n" +
         "   private Node parent;\n" +
         "   public Node node;\n" +
         "}\n");

      return buf.toString();
   }

   static String getNodeSequenceClassStr() {
      StringBuffer buf = new StringBuffer(
         "package " + packageName + ";\n\n" +
         "import java.util.*;\n\n" +
         "/**\n" +
         " * Represents a sequence of nodes nested within a choice, list,\n" +
         " * optional list, or optional, e.g. ( A B )+ or [ C D E ]\n" +
         " */\n"); 

      if ( Globals.superClassname == null )
         buf.append("public class NodeSequence implements NodeListInterface {\n");
      else
         buf.append("public class NodeSequence extends "+Globals.superClassname+" implements NodeListInterface {\n");

      buf.append(
         "   public NodeSequence(int n) {\n" +
         "      nodes = new Vector(n);\n" +
         "   }\n\n" +
         "   public NodeSequence(Node firstNode) {\n" +
         "      nodes = new Vector();\n" +
         "      addNode(firstNode);\n" +
         "   }\n\n" +
         "   public void addNode(Node n) {\n" +
         "      nodes.addElement(n);\n" +
         "      if (n != null) \n" +
         "         n.setParent(this);\n" +
         "      else \n"+
         "         throw new Error(\"Node is null.\"); \n" +
         "   }\n\n" +
         "   public Node elementAt(int i)  { return (Node)nodes.elementAt(i); }\n"+
         "   public Enumeration elements() { return nodes.elements(); }\n" +
         "   public int size()             { return nodes.size(); }\n\n" +
         "   public void accept(" + Globals.visitorPackage + ".Visitor v) {\n");

      if ( Globals.oldVisitNames )
         buf.append("      v.visitNodeSequence(this);\n");
      else
         buf.append("      v.visit(this);\n");

      buf.append(
         "   }\n\n" +
         "   public void setParent(Node n) { parent = n; }\n" +
         "   public Node getParent() { \n" +
         "      if (parent == null) throw new Error(\"Parent not set.\");\n"+
         "      return parent; \n"+
         "   }\n\n" +
         "   public void setChildAt(Node n, int i) { \n" +
         "      nodes.setElementAt(n, i); \n" +
         "      if (n != null) n.setParent(this); \n" +
         "   }\n" +
         "   public Vector getChildren() { \n" +
         "      return nodes; \n" +
         "   }\n\n" +
         "   private Node parent;\n" +
         "   public Vector nodes;\n" +
         "}\n");

      return buf.toString();
   }

   static String getNodeTokenClassStr() {
      StringBuffer buf = new StringBuffer(
         "package " + packageName + ";\n\n" +
         "import java.util.*;\n" +
         "/**\n" +
         " * Represents a single token in the grammar.  If the \"-tk\" option\n" +
         " * is used, also contains a Vector of preceding special tokens.\n" +
         " */\n");

      if ( Globals.superClassname == null )
         buf.append("public class NodeToken implements Node {\n");
      else
         buf.append("public class NodeToken extends "+Globals.superClassname+" implements Node {\n");

      buf.append(
         "   public NodeToken(String s) {\n" +
         "      this(s, -1, -1, -1, -1, -1); " +
         "   }\n\n" +
         "   public NodeToken(String s, int kind, int beginLine, int beginColumn, int endLine, int endColumn) {\n" +
         "      tokenImage = s;\n" +
         "      specialTokens = null;\n" +
         "      this.kind = kind;\n" +
         "      this.beginLine = beginLine;\n" +
         "      this.beginColumn = beginColumn;\n" +
         "      this.endLine = endLine;\n" +
         "      this.endColumn = endColumn;\n" +
         "   }\n\n" +
         "   public NodeToken getSpecialAt(int i) {\n" +
         "      if ( specialTokens == null )\n" +
         "         throw new NoSuchElementException(\"No specials in token\");\n" +
         "      return (NodeToken)specialTokens.elementAt(i);\n" +
         "   }\n\n" +
         "   public int numSpecials() {\n" +
         "      if ( specialTokens == null ) return 0;\n" +
         "      return specialTokens.size();\n" +
         "   }\n\n" +
         "   public void addSpecial(NodeToken s) {\n" +
         "      if ( specialTokens == null ) specialTokens = new Vector();\n" +
         "      specialTokens.addElement(s);\n" +
         "      s.setParent(this);\n" +
         "   }\n\n" +
         "   public void trimSpecials() {\n" +
         "      if ( specialTokens == null ) return;\n" +
         "      specialTokens.trimToSize();\n" +
         "   }\n\n" +
         "   public String toString()     { return tokenImage; }\n\n" +
         "   public String withSpecials() {\n" +
         "      if ( specialTokens == null )\n" +
         "          return tokenImage;\n\n" +
         "       StringBuffer buf = new StringBuffer();\n\n" +
         "       for ( Enumeration e = specialTokens.elements(); e.hasMoreElements(); )\n" +
         "          buf.append(e.nextElement().toString());\n\n" +
         "       buf.append(tokenImage);\n" +
         "       return buf.toString();\n" +
         "   }\n\n" +
         "   public void accept(" + Globals.visitorPackage + ".Visitor v) {\n");

      if ( Globals.oldVisitNames )
         buf.append("      v.visitNodeToken(this);\n");
      else
         buf.append("      v.visit(this);\n");

      buf.append(
         "   }\n\n" +
         "   public void setParent(Node n) { parent = n; }\n" +
         "   public Node getParent() { \n" +
         "      if (parent == null) throw new Error(\"Parent not set.\");\n"+
         "      return parent; \n"+
         "   }\n\n" +
         "   public void setChildAt(Node n, int i) { \n" +
         "      throw new Error(\"No children for NodeToken.\");  \n" +
         "   }\n" +
         "   public Vector getChildren() { \n" +
         "      throw new Error(\"No children for NodeToken.\");  \n" +
         "   }\n\n" +
         "   private Node parent;\n" +
         "   public String tokenImage;\n\n" +
         "   // Stores a list of NodeTokens\n" +
         "   public Vector specialTokens;\n\n" +
         "   // -1 for these ints means no position info is available.\n" +
         "   public int beginLine, beginColumn, endLine, endColumn;\n\n" +
         "   // Equal to the JavaCC token \"kind\" integer.\n" +
         "   // -1 if not available.\n" +
         "   public int kind;\n" +
         "}\n");

      return buf.toString();
   }

   //
   // The visitor methods for the auto classes
   //
   static String getNodeListVisitorStr() {
      if ( Globals.oldVisitNames )
         return "   public void visitNodeList(NodeList n);";
      else
         return "   public void visit(NodeList n);";
   }

   static String getNodeListOptionalVisitorStr() {
      if ( Globals.oldVisitNames )
         return "   public void visitNodeListOptional(NodeListOptional n);";
      else
         return "   public void visit(NodeListOptional n);";
   }

   static String getNodeOptionalVisitorStr() {
      if ( Globals.oldVisitNames )
         return "   public void visitNodeOptional(NodeOptional n);";
      else
         return "   public void visit(NodeOptional n);";
   }

   static String getNodeSequenceVisitorStr() {
      if ( Globals.oldVisitNames )
         return "   public void visitNodeSequence(NodeSequence n);";
      else
         return "   public void visit(NodeSequence n);";
   }

   static String getNodeTokenVisitorStr() {
      if ( Globals.oldVisitNames )
         return "   public void visitNodeToken(NodeToken n);";
      else
         return "   public void visit(NodeToken n);";
   }
}
