/*
 * All files in the distribution of JTB, The Java Tree Builder are
 * Copyright 1997, 1998 by the Purdue Research Foundation of Purdue
 * University.  All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted
 * provided that this entire copyright notice is duplicated in all
 * such copies, and that any documentation, announcements, and
 * other materials related to such distribution and use acknowledge
 * that the software was developed at Purdue University, West Lafayette,
 * Indiana by Kevin Tao and Jens Palsberg.  No charge may be made
 * for copies, derivations, or distributions of this material
 * without the express written consent of the copyright holder.
 * Neither the name of the University nor the name of the author
 * may be used to endorse or promote products derived from this
 * material without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR ANY PARTICULAR PURPOSE.
 */

package EDU.purdue.jtb.misc;

/**
 * Class Globals contains global program information.
 */
public class Globals {
   //
   // Global constants
   //
   public static final String progName = "JTB";
   public static final String scriptName = "jtb";
   public static final String version = "1.1.1";
   public static final String author = "Kevin Tao";
   public static final String email = "taokr@cs.purdue.edu";

   public static String fileHeader() {
      return fileHeader(new Spacing(3));
   }
   public static String fileHeader(Spacing spc) {
      return
//          "//\n" +
//          spc.spc + "// Generated by " + progName + " " + version + "\n" +
//          spc.spc + "//";
         "/** \n"+
         " * Author       : Guansong Zhang \n" +
         " * Revision     : $"+"Revision: $ \n" +
         " * Updated      : $"+"Date: $ \n" +
         " * Node         : Generated from " + progName + " " + version + " modified by author. \n" +
         " * Copyright    : All rights reserved \n" +
         " */";

   }

   //
   // Internal name constants
   //
   // TO DO -- remove all hard-coded names from source files.
   //
   public static final String choiceName = "NodeChoice";
   public static final String listName = "NodeList";
   public static final String listOptName = "NodeListOptional";
   public static final String optionalName = "NodeOptional";
   public static final String sequenceName = "NodeSequence";
   public static final String tokenName = "NodeToken";

   public static final String visitorName = "Visitor";

   //
   // Changeable flags
   //
   public static boolean printClassList = false;
   public static boolean generateFiles = true;
   public static boolean printGrammarToOut = false;
   public static boolean oldVisitNames = false;
   public static boolean keepSpecialTokens = false;
   public static boolean javaDocComments = false;
   public static boolean noOverwrite = false;
   public static boolean noSemanticCheck = false;

   public static boolean schemeToolkit = false;
   public static boolean printerToolkit = false;

   //
   // Changeable names
   //
   public static String nodePackage = "syntaxtree";
   public static String visitorPackage = "visitor";
   public static String nodeDir = "syntaxtree";
   public static String visitorDir = "visitor";

   public static String inFilename;                   // must be set in main()
   public static String outFilename = "jtb.out.jj";
   public static String superClassname = null;

   //
   // These two methods should be called rather than setting nodePackage
   // and visitorPackage directly.
   //
   public static void setNodePackage(String s) {
      int lastDotIndex = -1;
      nodePackage = s;
      
      for ( int i = 0; i < s.length(); ++i )
         if ( s.charAt(i) == '.' ) lastDotIndex = i;

      if ( lastDotIndex != -1 )
         nodeDir = s.substring(lastDotIndex + 1);
      else
         nodeDir = s;
   }
   
   public static void setVisitorPackage(String s) {
      int lastDotIndex = -1;
      visitorPackage = s;
      
      for ( int i = 0; i < s.length(); ++i )
         if ( s.charAt(i) == '.' ) lastDotIndex = i;

      if ( lastDotIndex != -1 )
         visitorDir = s.substring(lastDotIndex + 1);
      else
         visitorDir = s;
   }
}
