/*
 * All files in the distribution of JTB, The Java Tree Builder are
 * Copyright 1997, 1998 by the Purdue Research Foundation of Purdue
 * University.  All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted
 * provided that this entire copyright notice is duplicated in all
 * such copies, and that any documentation, announcements, and
 * other materials related to such distribution and use acknowledge
 * that the software was developed at Purdue University, West Lafayette,
 * Indiana by Kevin Tao and Jens Palsberg.  No charge may be made
 * for copies, derivations, or distributions of this material
 * without the express written consent of the copyright holder.
 * Neither the name of the University nor the name of the author
 * may be used to endorse or promote products derived from this
 * material without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR ANY PARTICULAR PURPOSE.
 */

//
// Class ExpansionUnitTypeCounter counts the types of the expansion units
// found in this subtree, without descending further into the tree than this
// level.  Typically will be executed on n.f0 of an Expansion.
//

package EDU.purdue.jtb.visitor;

import EDU.purdue.jtb.syntaxtree.*;
import EDU.purdue.jtb.misc.Errors;

public class ExpansionUnitTypeCounter extends DepthFirstVisitor {
   private int numLookaheads = 0;
   private int numBlocks = 0;
   private int numGroupings = 0;
   private int numTerms = 0;

   public int getNumNormals()     { return numGroupings + numTerms; }
   public int getNumLookaheads()  { return numLookaheads; }
   public int getNumBlocks()      { return numBlocks; }
   public int getNumGroupings()   { return numGroupings; }
   public int getNumTerms()       { return numTerms; }

   public void visit(ExpansionUnit n) {
      switch (n.f0.which) {
         case 0 : ++numLookaheads;  break;
         case 1 : ++numBlocks;      break;
         case 2 : // fall through
         case 3 : ++numGroupings;   break;
         case 4 : ++numTerms;       break;
         default: Errors.hardErr("which = " + String.valueOf(n.f0.which));
      }
   }
}
